import { Component, Directive, ViewContainerRef, ElementRef, ComponentRef, ComponentFactoryResolver } from '@angular/core';
import { CustomComponent } from './custom.component';
import { Column } from "igniteui-angular2";
import { Northwind } from "./northwind";

declare var jQuery: any;
declare var window: any;


/*extend the grid Column to MyColumn. Here the templating of IgniteUI widgets is overrided.
Once that's done you can build custom component based on each record.*/
@Directive({
    selector: 'my-column',
    inputs: ['key', 'headerText', 'template', 'width', 'dataType'],
    providers: [{ provide: Column, useExisting: MyColumn }]
})
export class MyColumn extends Column {
    cmpRef: ComponentRef<CustomComponent>;
    factory: any;

    constructor(private componentFactoryResolver: ComponentFactoryResolver, private viewContainerRef: ViewContainerRef, el: ElementRef) {
        super(el);
    }

    ngAfterViewInit() {
        var that = this;
        if (window.cmpRefs === undefined) {
            window.cmpRefs = {};
        }

        function template(tmpl, data) {
            
            if (window.cmpRefs[data["ProductID"]] === undefined) {
                that.factory = that.componentFactoryResolver.resolveComponentFactory(CustomComponent);
                //create CustomCoponent for each record
                let cmpRef = that.viewContainerRef.createComponent(that.factory);
                //store these components, they need to be stored once, to retain their instance state
                window.cmpRefs[data["ProductID"]] = cmpRef;
            }
            return '';
        }
        //Override IgniteUI Templating
        jQuery.ig.tmpl = template;
    }
}


@Component({
    selector: 'my-app',
    template: `<h1 id='mainheader'>My IgniteUI Angular2 App</h1>
    <div class="row">
        <div class="col-md-8">
            <ig-grid [widgetId]='id' (dataRendered)='renderedCallback($event)' [width]='w' [autoCommit]='true' [(dataSource)]='data' [height]='h' [autoGenerateColumns]='false' [primaryKey]='"ProductID"' [features]='features'>
                <column [key]="'ProductID'" [headerText]="'ProductID'" [width]="'165px'" [dataType]="'number'"></column>
                <column [key]="'ProductName'" [headerText]="'Product Name'" [width]="'150px'" [dataType]="'string'"></column>
                <column [key]="'QuantityPerUnit'" [headerText]="'Quantity per unit'" [width]="'150px'" [dataType]="'string'"></column>
                <column [key]="'UnitPrice'" [headerText]="'Unit Price'" [width]="'100px'" [dataType]="'number'"></column>
                <my-column [key]="'CustomComponent'" [headerText]="'Custom component'" [width]="'200px'" [dataType]="'number'" [template]="'<custom-component></custom-component>'"></my-column>
            </ig-grid>
        </div>
    </div>
    `
})
export class AppComponent {
	private gridOptions: IgGrid;
	private id: string;
	private data: any;
	private features: Array<any>;

	constructor() {
		this.data = Northwind.getData();

		this.id ='grid1';
		this.features = [{name: "Sorting"}];
	}

	renderedCallback($event) {
		let customComponents = window.cmpRefs;
		let id, nativeElement;
		if (customComponents !== undefined) {
            //when the grid is rendered use the stored component references to populate the custom component
			jQuery($event.ui.owner.element)
				.find("tbody td[aria-describedby='grid1_CustomComponent']")
				.each((index, element)=> {
					id = jQuery(element).parent().data('id');
					nativeElement = jQuery(customComponents[id].location.nativeElement);
					//append the native element of you custom component into the cell
					nativeElement.appendTo(element);
				});
		}
		
	}
}
